#------------------------------------------------------------------------------
# Copyright (c) 2017, Nucleic Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
#------------------------------------------------------------------------------
from .autumn import AUTUMN_THEME
from .colorful import COLORFUL_THEME
from .default import DEFAULT_THEME
from .emacs import EMACS_THEME
from .friendly import FRIENDLY_THEME
from .fruity import FRUITY_THEME
from .idle import IDLE_THEME
from .manni import MANNI_THEME
from .monokai import MONOKAI_THEME
from .murphy import MURPHY_THEME
from .native import NATIVE_THEME
from .pastie import PASTIE_THEME
from .perldoc import PERLDOC_THEME
from .tango import TANGO_THEME
from .trac import TRAC_THEME
from .vim import VIM_THEME


THEMES = {
    'autumn': AUTUMN_THEME,
    'colorful': COLORFUL_THEME,
    'default': DEFAULT_THEME,
    'emacs': EMACS_THEME,
    'friendly': FRIENDLY_THEME,
    'fruity': FRUITY_THEME,
    'idle': IDLE_THEME,
    'manni': MANNI_THEME,
    'monokai': MONOKAI_THEME,
    'murphy': MURPHY_THEME,
    'native': NATIVE_THEME,
    'pastie': PASTIE_THEME,
    'perldoc': PERLDOC_THEME,
    'tango': TANGO_THEME,
    'trac': TRAC_THEME,
    'vim': VIM_THEME,
}
